// Filename: XmlTester.h
// S.Chan, Jul 17, 2005

#ifndef XML_TESTER_INCLUDED
#define XML_TESTER_INCLUDED

// Include XML wrapper framework.
#include "XmlParser.h"

// Include STL string.
#include <string>

// CXmlTester
// Helper class for CTestXmlSupportDlg that
// uses CXmlParser to implement the various
// GUI options such as schema validation.
class CXmlTester : public IXmlElementHandler
{
public:
    CXmlTester();
    ~CXmlTester();

    // IXmlElementHandler virtual overrides.
    virtual void OnXmlStartElement(const CXmlElement& xmlElement);
    virtual void OnXmlElementData(const std::string& elementData, int depth);
    virtual void OnXmlEndElement(const CXmlElement& xmlElement);
    virtual void OnXmlError(int line, int column, const std::string& errorText, unsigned long errorCode);
    virtual bool OnXmlAbortParse(const CXmlElement& xmlElement);

    // Attach/detach results window for displaying parsing output.
    void AttachResultsWnd(CListBox* resultsWnd);
    void DetachResultsWnd();

    // Parsing control flow.
    void SetParsingDelay(int parsingDelay);
    void SetAbort(bool abort);
    
    // Set up parser features.
    void EnableSchemaValidation(bool enable);
    void EnableSchemaLocation(bool enable);
    void EnableExhaustiveErrors(bool enable);
    
    // Add/remove validation schema.
    bool AddValidationSchema(const std::string& namespaceURI, const std::string& xsdPath);
    bool RemoveValidationSchema(const std::string& namespaceURI);

    // Parse XML file.
    bool ParseXml(const std::string& xmlPath);

private:
    void        DisplayParseError(const std::string& errorText);

    CListBox*   m_resultsWnd;

    int         m_parsingDelay;     // Units in milliseconds.
    bool        m_abort;

    bool        m_enableSchemaValidation;
    bool        m_enableSchemaLocation;
    bool        m_enableExhaustiveErrors;

    // XML parser (wrapper class for SAX2).
    CXmlParser* m_xmlParser;
};
  
#endif // XML_TESTER_INCLUDED

// END
